------------------------------------------------------------------
-- title        flashprog_xs3_cpld
-- Version      1
-- Date         2007/05/14
-- 
-- Copyright    Copyright (c) 2006
-- Author       Stephan Schirrmann
-- Company      Simple Solutions
--
-- Description:
--   Simple CPLD code for Zefant XS3 on TopDown Baseboard
--
-- Notes:
--   fpga_cpld_io pinout:
--     (0) cpld_led(0)      - FROM FPGA!
--     (1) cpld_led(1)      - FROM FPGA!
--     (2) cpld_button(1)   - TO FPGA!   (typically used as rst_n)
--     (3) 
--
--     (4) bb_button(0)     - TO FPGA
--     (5) bb_button(1)     - TO FPGA
--     (6) bb_button(2)     - TO FPGA
--     (7) bb_button(3)     - TO FPGA
--
--
---------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity flashprog_xs3_cpld is
  port (
    -- Clock
    osc1_en           : out std_logic;
    osc1              : in  std_logic;
    
    -- Buttons / Switches / LEDs 
    cpld_led          : out std_logic_vector( 1 downto 0 );
    cpld_button       : in  std_logic_vector( 1 downto 0 );
    cpld_switch       : in  std_logic_vector( 1 downto 0 );

    -- Interface to FPGA
    fpga_cpld_io      : inout std_logic_vector( 7 downto 0 );

    -- Special function fpga pins 
    fpga_done         : in  std_logic;
    fpga_hswap_en     : out std_logic;
    fpga_m            : out std_logic_vector( 2 downto 0 );
    fpga_prog_b       : out std_logic;
    fpga_cclk         : out std_logic;
    fpga_cclk_dir     : out std_logic;
    fpga_cs_n         : out std_logic;
    fpga_init_n       : in std_logic;
    fpga_rdwr_n       : out std_logic;
    fpga_dout_busy    : in  std_logic;

    -- Flash memory
    fl_a              : out   std_logic_vector(24 downto 0);
    fl_d              : inout std_logic_vector( 7 downto 0);
    fl_ce_n           : out   std_logic;
    fl_oe_n           : out   std_logic;
    fl_we_n           : out   std_logic;
    fl_rp_n           : out   std_logic;
    fl_byte_n         : out   std_logic;

    -- Boot PROM
    xcf_ce_n          : out   std_logic;
    xcf_cf_n          : in    std_logic;
    
    -- Power supply control
    psen_25_12        : out std_logic;

    -- Baseboard hardware
    rs232_from_fpga   : out std_logic;
    rs232_to_fpga     : in  std_logic;

    bb_button         : in std_logic_vector(3 downto 0)
    
  );
end flashprog_xs3_cpld;

architecture Behavioral of flashprog_xs3_cpld is

  type cpldStateType is ( ST_WAIT_FPGA_INIT, 

                          ST_START_BOOTING_PARALLEL, 
                          ST_BOOTING_PARALLEL,

                          ST_RUNNING );
                          
  signal cpld_state : cpldStateType := ST_WAIT_FPGA_INIT;
  
  type bootModeType is ( MO_BOOT_PARALLEL, 
                         MO_JTAG_ONLY, 
                         MO_BOOT_XCF );
  signal boot_mode : bootModeType := MO_BOOT_PARALLEL;
  
signal reboot_n  : std_logic;

signal fl_a_s    : std_logic_vector(24 downto 0);
signal fl_rp_n_s : std_logic;
signal fl_ce_n_s : std_logic;
signal fl_oe_n_s : std_logic;

signal clk       : std_logic;

begin
  
  -- FPGA-CPLD Connections ---------------------------------------------
  cpld_led(0)     <= not fpga_cpld_io(0) when fpga_done='1' else '0';
  cpld_led(1)     <= not fpga_cpld_io(1) when fpga_done='1' else '1';
  
  fpga_cpld_io(0)          <= 'Z';
  fpga_cpld_io(1)          <= 'Z';
  fpga_cpld_io(2)          <= cpld_button(1);
  fpga_cpld_io(3)          <= 'Z';
  fpga_cpld_io(7 downto 4) <= bb_button(3 downto 0);

 
  -- Main CPLD State Machine
  main_fsm : process ( clk, reboot_n )
  variable boot_addr : integer range 0 to 33554433;
  begin

    if reboot_n = '0' then
      cpld_state    <= ST_WAIT_FPGA_INIT;
      boot_mode     <= MO_BOOT_PARALLEL;

      fpga_cs_n     <= '1';
      fpga_rdwr_n   <= '1';

      boot_addr     := 0;
      fl_rp_n_s     <= '0';
      fl_ce_n_s     <= '1';
      fl_oe_n_s     <= '1';
      
    elsif rising_edge(clk) then
      case cpld_state is
        when ST_WAIT_FPGA_INIT =>
          if cpld_switch(0)='0' then 
            boot_mode   <= MO_BOOT_PARALLEL;
            cpld_state    <= ST_WAIT_FPGA_INIT;
            if fpga_init_n = '1' then
              cpld_state <= ST_START_BOOTING_PARALLEL;
              boot_addr  := 0;
              fl_rp_n_s  <= '1';
              fl_ce_n_s  <= '0';
              fl_oe_n_s  <= '0';
            end if;
          else
            boot_mode   <= MO_JTAG_ONLY;
          end if;

        -- Parallel selectMAP booting starts here
        when ST_START_BOOTING_PARALLEL =>
          fpga_cs_n   <= '0';
          fpga_rdwr_n <= '0';
          cpld_state  <= ST_BOOTING_PARALLEL;
          
        when ST_BOOTING_PARALLEL =>
          if fpga_init_n='1' and fpga_dout_busy='0' then
            boot_addr := boot_addr + 1;
          end if;

          fl_a_s <= conv_std_logic_vector(boot_addr, 25);
          
          if fpga_done='1' then 
            cpld_state <= ST_RUNNING;
          end if;
          
        when ST_RUNNING => 
          fpga_cs_n   <= 'Z';
          fpga_rdwr_n <= 'Z';
          
        when others =>
          cpld_state <= ST_WAIT_FPGA_INIT;
      end case;
    end if;
  end process;
   
  -- Set Flash ports
  set_fl_ports : process ( fpga_done, fl_a_s, fl_ce_n_s, fl_oe_n_s, fl_rp_n_s )
  begin
    if fpga_done='0' then
      fl_a      <= fl_a_s   ;
      fl_ce_n   <= fl_ce_n_s;
      fl_oe_n   <= fl_oe_n_s;
      fl_rp_n   <= fl_rp_n_s;
      fl_we_n   <= '1';
      fl_byte_n <= '0';
    else
      fl_a      <= (others => 'Z');
      fl_ce_n   <= 'Z';
      fl_oe_n   <= 'Z';
      fl_rp_n   <= 'Z';
      fl_we_n   <= 'Z';
      fl_byte_n <= 'Z';
    end if;
  end process;

  fl_d    <= (others => 'Z');

  -- Bootmode ----------------------------------------------------------
  bootmode : process (boot_mode, xcf_cf_n, fpga_done)
  begin
    
    if boot_mode = MO_BOOT_PARALLEL then
      fpga_m            <= "110";     -- "110" = Slave Parallel Mode
      fpga_prog_b       <= cpld_button(0); -- reset
      fpga_cclk_dir     <= '1';       -- '0' = use CCLK from FPGA 
                                      -- '1' = use CCLK from CPLD or BootPROM
      xcf_ce_n          <= '1';
    
--    elsif boot_mode = MO_BOOT_XCF then
--      fpga_m            <= "000";     -- "000" = Master Serial Mode
--      fpga_prog_b       <= xcf_cf_n;
--      fpga_cclk         <= 'Z';       -- CCLK is generated by FPGA
--      fpga_cclk_dir     <= '0';       -- '0' = use CCLK from FPGA 
--                                      -- '1' = use CCLK from CPLD or BootPROM
--      xcf_ce_n          <= fpga_done;

    else
      fpga_m            <= "101";          -- "101" = Boundary Scan Mode
      fpga_prog_b       <= cpld_button(0); -- reset
      fpga_cclk_dir     <= '1';            -- '0' = use CCLK from FPGA 
                                           -- '1' = use CCLK from CPLD or BootPROM
      xcf_ce_n          <= '1';
    end if;
  end process;


---------------------------------------------------------------------------------------------------
-- Clock divider
---------------------------------------------------------------------------------------------------

  clkdiv : process (reboot_n, osc1)
  variable counter : integer;
  begin
    if reboot_n = '0' then
      counter := 0;
      clk <= '0';
    elsif rising_edge(osc1) then
      counter := counter + 1;
      if counter = 5 then
        counter := 0;
        clk <= not clk; 
	    end if;
    end if;
  end process;


  fpga_cclk         <= clk;


  -- Signal renaming  --------------------------------------------------
  reboot_n  <= cpld_button(0);

  -- Special function fpga pins ----------------------------------------
  fpga_hswap_en     <= '1';       -- Configtime-Pullups '1' = disabled / '0' = enabled

  -- Misc configuration pins -------------------------------------------
  psen_25_12        <= '1';       -- Enable VIN/VAUX Power Supply
  osc1_en           <= '1';       -- Enable Oscillator

end Behavioral;
